@@ -13,11 +13,13 @@
 //     without express or implied warranty.
 ////////////////////////////////////////////////////////////////////////////////
 
-// $Header:
+// $Header: /cvsroot/loki-lib/loki/include/loki/Functor.h,v 1.22 2006/06/09 12:58:44 syntheticpp Exp $
 
 #ifndef LOKI_FUNCTOR_INC_
 #define LOKI_FUNCTOR_INC_
 
+// $Header: /cvsroot/loki-lib/loki/include/loki/Functor.h,v 1.22 2006/06/09 12:58:44 syntheticpp Exp $
+
 #include "Typelist.h"
 #include "Sequence.h"
 #include "EmptyType.h"
@@ -28,6 +30,17 @@
 
 ///  \defgroup FunctorGroup Function objects
 
+#ifndef LOKI_FUNCTOR_IS_NOT_A_SMALLOBJECT
+//#define LOKI_FUNCTOR_IS_NOT_A_SMALLOBJECT
+#endif
+
+#ifndef LOKI_FUNCTORS_ARE_COMPARABLE
+//#define LOKI_FUNCTORS_ARE_COMPARABLE
+#endif
+
+
+/// \namespace Loki
+/// All classes of Loki are in the Loki namespace
 namespace Loki
 {
 ////////////////////////////////////////////////////////////////////////////////
@@ -36,11 +49,22 @@ namespace Loki
 
     namespace Private
     {
-        template <typename R, template <class> class ThreadingModel>
-        struct FunctorImplBase : public SmallObject<ThreadingModel>
+        template <typename R, template <class, class> class ThreadingModel>
+        struct FunctorImplBase 
+#ifdef LOKI_FUNCTOR_IS_NOT_A_SMALLOBJECT
         {
+#else
+            : public SmallValueObject<ThreadingModel>
+        {
+            inline FunctorImplBase() :
+                SmallValueObject<ThreadingModel>() {}
+            inline FunctorImplBase(const FunctorImplBase&) :
+                SmallValueObject<ThreadingModel>() {}
+#endif
+
             typedef R ResultType;
-            
+            typedef FunctorImplBase<R, ThreadingModel> FunctorImplBaseType;
+
             typedef EmptyType Parm1;
             typedef EmptyType Parm2;
             typedef EmptyType Parm3;
@@ -57,10 +81,12 @@ namespace Loki
             typedef EmptyType Parm14;
             typedef EmptyType Parm15;
 
-            inline FunctorImplBase(){};
-            inline FunctorImplBase(const FunctorImplBase&){};
+
+            virtual ~FunctorImplBase()
+            {}
 
             virtual FunctorImplBase* DoClone() const = 0;
+
             template <class U>
             static U* Clone(U* pObj)
             {
@@ -69,6 +95,14 @@ namespace Loki
                 assert(typeid(*pClone) == typeid(*pObj));
                 return pClone;
             }
+
+
+#ifdef LOKI_FUNCTORS_ARE_COMPARABLE
+
+            virtual bool operator==(const FunctorImplBase&) const = 0;
+           
+#endif            
+         
         };
     }
     
@@ -90,7 +124,7 @@ namespace Loki
 ////////////////////////////////////////////////////////////////////////////////
 
     template <typename R, class TList, 
-        template <class> class ThreadingModel = LOKI_DEFAULT_THREADING_NO_OBJ_LEVEL>
+        template <class, class> class ThreadingModel = LOKI_DEFAULT_THREADING_NO_OBJ_LEVEL>
     class FunctorImpl;
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -98,7 +132,7 @@ namespace Loki
 // Specialization for 0 (zero) parameters
 ////////////////////////////////////////////////////////////////////////////////
 
-    template <typename R, template <class> class ThreadingModel>
+    template <typename R, template <class, class> class ThreadingModel>
     class FunctorImpl<R, NullType, ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
     {
@@ -112,7 +146,7 @@ namespace Loki
 // Specialization for 1 parameter
 ////////////////////////////////////////////////////////////////////////////////
 
-    template <typename R, typename P1, template <class> class ThreadingModel>
+    template <typename R, typename P1, template <class, class> class ThreadingModel>
         class FunctorImpl<R, Seq<P1>, ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
     {
@@ -128,7 +162,7 @@ namespace Loki
 ////////////////////////////////////////////////////////////////////////////////
 
     template <typename R, typename P1, typename P2, 
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R, Seq<P1, P2>, ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
     {
@@ -145,7 +179,7 @@ namespace Loki
 ////////////////////////////////////////////////////////////////////////////////
 
     template <typename R, typename P1, typename P2, typename P3,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R, Seq<P1, P2, P3>, ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
     {
@@ -163,7 +197,7 @@ namespace Loki
 ////////////////////////////////////////////////////////////////////////////////
 
     template <typename R, typename P1, typename P2, typename P3, typename P4,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R, Seq<P1, P2, P3, P4>, ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
     {
@@ -183,7 +217,7 @@ namespace Loki
 
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R, Seq<P1, P2, P3, P4, P5>, ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
     {
@@ -204,7 +238,7 @@ namespace Loki
 
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R, Seq<P1, P2, P3, P4, P5, P6>, ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
     {
@@ -226,7 +260,7 @@ namespace Loki
 
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6, typename P7,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R, Seq<P1, P2, P3, P4, P5, P6, P7>, ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
     {
@@ -250,7 +284,7 @@ namespace Loki
 
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6, typename P7, typename P8,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R, Seq<P1, P2, P3, P4, P5, P6, P7, P8>,
             ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
@@ -276,7 +310,7 @@ namespace Loki
 
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6, typename P7, typename P8, typename P9,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R, Seq<P1, P2, P3, P4, P5, P6, P7, P8, P9>,
             ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
@@ -304,7 +338,7 @@ namespace Loki
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6, typename P7, typename P8, typename P9,
         typename P10,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R, Seq<P1, P2, P3, P4, P5, P6, P7, P8, P9, P10>,
             ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
@@ -333,7 +367,7 @@ namespace Loki
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6, typename P7, typename P8, typename P9,
         typename P10, typename P11,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R,
             Seq<P1, P2, P3, P4, P5, P6, P7, P8, P9, P10, P11>,
             ThreadingModel>
@@ -364,7 +398,7 @@ namespace Loki
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6, typename P7, typename P8, typename P9,
         typename P10, typename P11, typename P12,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R,
             Seq<P1, P2, P3, P4, P5, P6, P7, P8, P9, P10, P11, P12>,
             ThreadingModel>
@@ -396,7 +430,7 @@ namespace Loki
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6, typename P7, typename P8, typename P9,
         typename P10, typename P11, typename P12, typename P13,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R,
             Seq<P1, P2, P3, P4, P5, P6, P7, P8, P9, P10, P11, P12, P13>,
             ThreadingModel>
@@ -429,7 +463,7 @@ namespace Loki
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6, typename P7, typename P8, typename P9,
         typename P10, typename P11, typename P12, typename P13, typename P14,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R,
             Seq<P1, P2, P3, P4, P5, P6, P7, P8, P9, P10, P11, P12, P13,
                 P14>,
@@ -464,7 +498,7 @@ namespace Loki
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6, typename P7, typename P8, typename P9,
         typename P10, typename P11, typename P12, typename P13, typename P14,
-        typename P15, template <class> class ThreadingModel>
+        typename P15, template <class, class> class ThreadingModel>
     class FunctorImpl<R,
             Seq<P1, P2, P3, P4, P5, P6, P7, P8, P9, P10, P11, P12, P13,
                 P14, P15>,
@@ -500,7 +534,7 @@ namespace Loki
 // Specialization for 1 parameter
 ////////////////////////////////////////////////////////////////////////////////
 
-    template <typename R, typename P1, template <class> class ThreadingModel>
+    template <typename R, typename P1, template <class, class> class ThreadingModel>
     class FunctorImpl<R, LOKI_TYPELIST_1(P1), ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
     {
@@ -516,7 +550,7 @@ namespace Loki
 ////////////////////////////////////////////////////////////////////////////////
 
     template <typename R, typename P1, typename P2, 
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R, LOKI_TYPELIST_2(P1, P2), ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
     {
@@ -533,7 +567,7 @@ namespace Loki
 ////////////////////////////////////////////////////////////////////////////////
 
     template <typename R, typename P1, typename P2, typename P3,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R, LOKI_TYPELIST_3(P1, P2, P3), ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
     {
@@ -551,7 +585,7 @@ namespace Loki
 ////////////////////////////////////////////////////////////////////////////////
 
     template <typename R, typename P1, typename P2, typename P3, typename P4,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R, LOKI_TYPELIST_4(P1, P2, P3, P4), ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
     {
@@ -571,7 +605,7 @@ namespace Loki
 
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R, LOKI_TYPELIST_5(P1, P2, P3, P4, P5), ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
     {
@@ -592,7 +626,7 @@ namespace Loki
 
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R, LOKI_TYPELIST_6(P1, P2, P3, P4, P5, P6), ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
     {
@@ -614,7 +648,7 @@ namespace Loki
 
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6, typename P7,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R, LOKI_TYPELIST_7(P1, P2, P3, P4, P5, P6, P7), ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
     {
@@ -638,7 +672,7 @@ namespace Loki
 
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6, typename P7, typename P8,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R, LOKI_TYPELIST_8(P1, P2, P3, P4, P5, P6, P7, P8),
             ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
@@ -664,7 +698,7 @@ namespace Loki
 
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6, typename P7, typename P8, typename P9,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R, LOKI_TYPELIST_9(P1, P2, P3, P4, P5, P6, P7, P8, P9),
             ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
@@ -692,7 +726,7 @@ namespace Loki
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6, typename P7, typename P8, typename P9,
         typename P10,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R, LOKI_TYPELIST_10(P1, P2, P3, P4, P5, P6, P7, P8, P9, P10),
             ThreadingModel>
         : public Private::FunctorImplBase<R, ThreadingModel>
@@ -721,7 +755,7 @@ namespace Loki
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6, typename P7, typename P8, typename P9,
         typename P10, typename P11,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R,
             LOKI_TYPELIST_11(P1, P2, P3, P4, P5, P6, P7, P8, P9, P10, P11),
             ThreadingModel>
@@ -752,7 +786,7 @@ namespace Loki
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6, typename P7, typename P8, typename P9,
         typename P10, typename P11, typename P12,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R,
             LOKI_TYPELIST_12(P1, P2, P3, P4, P5, P6, P7, P8, P9, P10, P11, P12),
             ThreadingModel>
@@ -784,7 +818,7 @@ namespace Loki
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6, typename P7, typename P8, typename P9,
         typename P10, typename P11, typename P12, typename P13,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R,
             LOKI_TYPELIST_13(P1, P2, P3, P4, P5, P6, P7, P8, P9, P10, P11, P12, P13),
             ThreadingModel>
@@ -817,7 +851,7 @@ namespace Loki
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6, typename P7, typename P8, typename P9,
         typename P10, typename P11, typename P12, typename P13, typename P14,
-        template <class> class ThreadingModel>
+        template <class, class> class ThreadingModel>
     class FunctorImpl<R,
             LOKI_TYPELIST_14(P1, P2, P3, P4, P5, P6, P7, P8, P9, P10, P11, P12, P13,
                 P14),
@@ -852,7 +886,7 @@ namespace Loki
     template <typename R, typename P1, typename P2, typename P3, typename P4,
         typename P5, typename P6, typename P7, typename P8, typename P9,
         typename P10, typename P11, typename P12, typename P13, typename P14,
-        typename P15, template <class> class ThreadingModel>
+        typename P15, template <class, class> class ThreadingModel>
     class FunctorImpl<R,
             LOKI_TYPELIST_15(P1, P2, P3, P4, P5, P6, P7, P8, P9, P10, P11, P12, P13,
                 P14, P15),
@@ -916,6 +950,23 @@ namespace Loki
         
         LOKI_DEFINE_CLONE_FUNCTORIMPL(FunctorHandler)
 
+
+#ifdef LOKI_FUNCTORS_ARE_COMPARABLE
+
+
+        bool operator==(const typename Base::FunctorImplBaseType& rhs) const
+        {
+            // there is no static information if Functor holds a member function 
+            // or a free function; this is the main difference to tr1::function
+            if(typeid(*this) != typeid(rhs))
+                return false; // cannot be equal
+
+            const FunctorHandler& fh = static_cast<const FunctorHandler&>(rhs);
+            // if this line gives a compiler error, you are using a function object.
+            // you need to implement bool MyFnObj::operator == (const MyFnObj&) const;
+            return  f_==fh.f_;
+        }
+#endif
         // operator() implementations for up to 15 arguments
                 
         ResultType operator()()
@@ -1024,7 +1075,22 @@ namespace Loki
         {}
         
         LOKI_DEFINE_CLONE_FUNCTORIMPL(MemFunHandler)
-        
+
+
+#ifdef LOKI_FUNCTORS_ARE_COMPARABLE
+
+        bool operator==(const typename Base::FunctorImplBaseType& rhs) const
+        {
+            if(typeid(*this) != typeid(rhs))
+                return false; // cannot be equal 
+
+            const MemFunHandler& mfh = static_cast<const MemFunHandler&>(rhs);
+            // if this line gives a compiler error, you are using a function object.
+            // you need to implement bool MyFnObj::operator == (const MyFnObj&) const;
+            return  pObj_==mfh.pObj_ && pMemFn_==mfh.pMemFn_;
+        }
+#endif   
+
         ResultType operator()()
         { return ((*pObj_).*pMemFn_)(); }
 
@@ -1133,9 +1199,25 @@ namespace Loki
 ///
 ///  \ingroup FunctorGroup
 ///  A generalized functor implementation with value semantics
+///
+/// \par Macro: LOKI_FUNCTOR_IS_NOT_A_SMALLOBJECT
+/// Define 
+/// \code LOKI_FUNCTOR_IS_NOT_A_SMALLOBJECT \endcode
+/// to avoid static instantiation/delete 
+/// order problems.
+/// It often helps against crashes when using static Functors and multi threading.
+/// Defining also removes problems when unloading Dlls which hosts
+/// static Functor objects.
+///
+/// \par Macro: LOKI_FUNCTORS_ARE_COMPARABLE
+/// To enable the operator== define the macro
+/// \code LOKI_FUNCTORS_ARE_COMPARABLE \endcode
+/// The macro is disabled by default, because it breaks compiling functor 
+/// objects  which have no operator== implemented, keep in mind when you enable
+/// operator==.
 ////////////////////////////////////////////////////////////////////////////////
     template <typename R = void, class TList = NullType,
-        template<class> class ThreadingModel = LOKI_DEFAULT_THREADING_NO_OBJ_LEVEL>
+        template<class, class> class ThreadingModel = LOKI_DEFAULT_THREADING_NO_OBJ_LEVEL>
     class Functor
     {
     public:
@@ -1210,6 +1292,26 @@ namespace Loki
         }
 #endif
 
+#ifdef LOKI_FUNCTORS_ARE_COMPARABLE
+
+        bool operator==(const Functor& rhs) const
+        {
+            if(spImpl_.get()==0 && rhs.spImpl_.get()==0)
+                return true;
+            if(spImpl_.get()!=0 && rhs.spImpl_.get()!=0)
+                return *spImpl_.get() == *rhs.spImpl_.get();
+            else
+                return false;
+        }
+
+        bool operator!=(const Functor& rhs) const
+        {
+            return !(*this==rhs);
+        }
+#endif
+
+        // operator() implementations for up to 15 arguments
+
         ResultType operator()() const
         {
             LOKI_FUNCTION_THROW_BAD_FUNCTION_CALL
@@ -1328,11 +1430,18 @@ namespace Loki
         std::auto_ptr<Impl> spImpl_;
     };
     
+
+////////////////////////////////////////////////////////////////////////////////
+//  
+//  BindersFirst and Chainer 
+//
+////////////////////////////////////////////////////////////////////////////////
+
     namespace Private
     {
         template <class Fctor> struct BinderFirstTraits;
 
-        template <typename R, class TList, template <class> class ThreadingModel>
+        template <typename R, class TList, template <class, class> class ThreadingModel>
         struct BinderFirstTraits< Functor<R, TList, ThreadingModel> >
         {
             typedef Functor<R, TList, ThreadingModel> OriginalFunctor;
@@ -1359,7 +1468,7 @@ namespace Loki
             typedef typename TypeTraits<T>::ParameterType RefOrValue;
         };
         
-        template <typename R, class TList, template <class> class ThreadingModel>
+        template <typename R, class TList, template <class, class> class ThreadingModel>
         struct BinderFirstBoundTypeStorage< Functor<R, TList, ThreadingModel> >
         {
             typedef Functor<R, TList, ThreadingModel> OriginalFunctor;
@@ -1414,7 +1523,20 @@ namespace Loki
         {}
 
         LOKI_DEFINE_CLONE_FUNCTORIMPL(BinderFirst)
+
+#ifdef LOKI_FUNCTORS_ARE_COMPARABLE
         
+        bool operator==(const typename Base::FunctorImplBaseType& rhs) const
+        {
+            if(typeid(*this) != typeid(rhs))
+                return false; // cannot be equal 
+            // if this line gives a compiler error, you are using a function object.
+            // you need to implement bool MyFnObj::operator == (const MyFnObj&) const;
+            return    f_ == ((static_cast<const BinderFirst&> (rhs)).f_) &&
+                      b_ == ((static_cast<const BinderFirst&> (rhs)).b_);
+        }
+#endif
+
         // operator() implementations for up to 15 arguments
                 
         ResultType operator()()
@@ -1534,6 +1656,19 @@ namespace Loki
 
         LOKI_DEFINE_CLONE_FUNCTORIMPL(Chainer)
 
+#ifdef LOKI_FUNCTORS_ARE_COMPARABLE
+                
+        bool operator==(const typename Base::Impl::FunctorImplBaseType& rhs) const
+        {
+            if(typeid(*this) != typeid(rhs))
+                return false; // cannot be equal 
+            // if this line gives a compiler error, you are using a function object.
+            // you need to implement bool MyFnObj::operator == (const MyFnObj&) const;
+            return    f1_ == ((static_cast<const Chainer&> (rhs)).f2_) &&
+                      f2_ == ((static_cast<const Chainer&> (rhs)).f1_);
+        }
+#endif
+
         // operator() implementations for up to 15 arguments
 
         ResultType operator()()
@@ -1658,3 +1793,33 @@ namespace Loki
 ////////////////////////////////////////////////////////////////////////////////
 
 #endif  // FUNCTOR_INC_
+
+// $Log: Functor.h,v $
+// Revision 1.22  2006/06/09 12:58:44  syntheticpp
+// simplify Functor::operator== implementation. Thanks to Eric Beyeler
+//
+// Revision 1.21  2006/06/01 12:33:05  syntheticpp
+// add operator== to Functor, initiated by Eric Beyeler
+//
+// Revision 1.20  2006/05/20 10:23:07  syntheticpp
+// add warnings in the documentation about the special lifetime when using SmallObjects
+//
+// Revision 1.19  2006/03/08 18:22:42  syntheticpp
+// doxygen fixes
+//
+// Revision 1.18  2006/02/27 18:53:41  syntheticpp
+// make it possible for Functor not to be a small object because it introduces sometimes problems, add documentation
+//
+// Revision 1.17  2006/02/20 16:08:32  syntheticpp
+// gcc needs the template parameter, gcc warns about missing virtual destructor (because there is already a virtual function)
+//
+// Revision 1.16  2006/02/18 19:37:40  rich_sposato
+// Added explicit call to base copy-constructor.  Changed base class from
+// SmallObject to SmallValueObject.
+//
+// Revision 1.15  2006/01/22 13:31:45  syntheticpp
+// add additional template parameter for the changed threading classes
+//
+// Revision 1.14  2006/01/16 19:05:09  rich_sposato
+// Added cvs keywords.
+//
